(*****************************************************************************
 *
 * Code: Convert CPN Occurrence Graph to FSM
 * Project: CPN
 *
 * Author: Steven Gordon
 *         Sirindhorn International Institute of Technology
 *         Thammasat University
 *         Bangkok, Thailand
 *         steven.gordon@ieee.org
 *
 * Description:
 * Using CPNTools, generic code for converting an Occurrence Graph to a 
 * Finite State Machine, stored as a text file. The user needs to define
 * the functions that map binding elements to FSM transitions, as well as 
 * the halt states.
 *
 * $Revision$
 * $Author$
 * $Date$
 * $URL$
 * 
 ****************************************************************************)

(* Write OG to text file in the format: srcndoeno destndoeno primitveno *)
(* (Arc -> string) -> string -> unit *)
fun og2fsmtrans arc2fsm filename =
let
    val outfile = TextIO.openOut(filename);
    fun getfsminput (a) = TextIO.output(outfile, st_Node(SourceNode(a))^" "^
					st_Node(DestNode(a))^" "^
					arc2fsm(a)^"\n")
in
    EvalAllArcs(getfsminput);
    TextIO.closeOut(outfile)
end;


(* Calculate halt states for FSM and write to text file, one per line *)
(* (Node -> bool) -> string -> unit *)
fun og2fsmhalts findhalts filename =
let
    val outfile = TextIO.openOut(filename)
    fun WriteNodes(n) = TextIO.output(outfile, st_Node(n) ^ "\n")
in
    SearchAllNodes(findhalts, fn n => WriteNodes(n),[],op ::);
    TextIO.closeOut(outfile)
end;
